<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xslout="can be anything, doesn't matter">

  <xsl:output method="xml" indent="yes"/>

  <xsl:namespace-alias stylesheet-prefix="xslout" result-prefix="xsl"/>

  <xsl:template match="*|@*|text()|comment()|processing-instruction()">
    <xsl:copy>
      <xsl:apply-templates 
        select="*|@*|text()|comment()|processing-instruction()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="for-loop">
    <xslout:call-template name="for-loop">
      <xslout:with-param name="i" select="{@index-variable}"/>
      <xslout:with-param name="increment" select="{@increment}"/>
      <xslout:with-param name="operator">
        <xsl:attribute name="select">
          <xsl:text>'</xsl:text>
          <xsl:value-of select="@operator"/>
          <xsl:text>'</xsl:text>
        </xsl:attribute>
      </xslout:with-param>
      <xslout:with-param name="testValue" select="{@test-value}"/>
    </xslout:call-template>
  </xsl:template>

  <xsl:template match="for-loop" mode="generate-template">
    <xslout:variable name="newline">
<xslout:text>
</xslout:text>
    </xslout:variable>

    <xslout:template name="for-loop">
      <xslout:param name="i" select="@index-variable"/>
      <xslout:param name="increment" select="@increment"/>
      <xslout:param name="operator" select="@operator"/>
      <xslout:param name="testValue" select="@test-value"/>
      <xslout:param name="iteration" select="1"/>

      <xslout:variable name="testPassed">
        <xslout:choose>
          <xslout:when test="starts-with($operator, '!=')">
            <xslout:if test="$i != $testValue">
              <xslout:text>true</xslout:text>
            </xslout:if>
          </xslout:when>
          <xslout:when test="starts-with($operator, '&lt;=')">
            <xslout:if test="$i &lt;= $testValue">
              <xslout:text>true</xslout:text>
            </xslout:if>
          </xslout:when>
          <xslout:when test="starts-with($operator, '&gt;=')">
            <xslout:if test="$i &gt;= $testValue">
              <xslout:text>true</xslout:text>
            </xslout:if>
          </xslout:when>
          <xslout:when test="starts-with($operator, '=')">
            <xslout:if test="$i = $testValue">
              <xslout:text>true</xslout:text>
            </xslout:if>
          </xslout:when>
          <xslout:when test="starts-with($operator, '&lt;')">
            <xslout:if test="$i &lt; $testValue">
              <xslout:text>true</xslout:text>
            </xslout:if>
          </xslout:when>
          <xslout:when test="starts-with($operator, '&gt;')">
            <xslout:if test="$i &gt; $testValue">
              <xslout:text>true</xslout:text>
            </xslout:if>
          </xslout:when>
          <xslout:otherwise>
            <xslout:message terminate="yes">
              <xslout:text>Sorry, the for-loop emulator only </xslout:text>
              <xslout:text>handles six operators </xslout:text>
              <xslout:value-of select="$newline"/>
              <xslout:text>(&lt; | &gt; | = | &lt;= | &gt;= | !=).  </xslout:text>
              <xslout:text>The value </xslout:text>
              <xslout:value-of select="$operator"/>
              <xslout:text> is not allowed.</xslout:text>
              <xslout:value-of select="$newline"/>
            </xslout:message>
          </xslout:otherwise>
        </xslout:choose>
      </xslout:variable>

      <xslout:if test="$testPassed='true'">
        <xslout:comment>From your stylesheet:</xslout:comment>

        <xsl:apply-templates select="*"/>

        <xslout:comment>End of text from your stylesheet</xslout:comment>

        <xslout:call-template name="for-loop">
          <xslout:with-param name="i" select="$i + $increment"/>
          <xslout:with-param name="increment" select="$increment"/>
          <xslout:with-param name="operator" select="$operator"/>
          <xslout:with-param name="testValue" select="$testValue"/>
          <xslout:with-param name="iteration" select="$iteration + 1"/>
        </xslout:call-template>
      </xslout:if>
    </xslout:template>
  </xsl:template>

  <xsl:template match="/">
    <xslout:stylesheet version="1.0">
      <xsl:apply-templates select="//for-loop" mode="generate-template"/>
      <xslout:template match="/">
        <xsl:apply-templates select="*"/>
      </xslout:template>
    </xslout:stylesheet>
  </xsl:template>

</xsl:stylesheet>
