<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html" indent="yes"/>
<xsl:strip-space elements="*"/>

  <xsl:key name="language-index" match="defn" use="@language"/>
  <xsl:key name="term-ids"       match="term" use="@id"/>

  <xsl:template match="/">
    <xsl:apply-templates select="glossary"/>
  </xsl:template>

  <xsl:template match="glossary">
    <html>
      <head>
        <title>
          <xsl:text>Glossary Listing: </xsl:text>
          <xsl:value-of select="glentry[1]/term"/>
          <xsl:text> - </xsl:text>
          <xsl:value-of select="glentry[last()]/term"/>
        </title>
      </head>
      <body>
        <h1>
          <xsl:text>Glossary Listing: </xsl:text>
          <xsl:value-of select="glentry[1]/term"/>
          <xsl:text> - </xsl:text>
          <xsl:value-of select="glentry[last()]/term"/>
        </h1>
        <xsl:apply-templates select="glentry"/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="glentry">
    <p>
      <b>
        <a name="{term/@id}"/>
        <xsl:value-of select="term"/>
        <xsl:text>: </xsl:text>
      </b>
      <xsl:apply-templates select="defn"/>
    </p>
  </xsl:template>

  <xsl:template match="defn">
    <xsl:apply-templates 
     select="*|comment()|processing-instruction()|text()"/>
  </xsl:template>

  <xsl:template match="xref">
    <a href="#{@refid}">
      <xsl:choose>
        <xsl:when test="key('term-ids', @refid)[1]/@xreftext">
          <xsl:value-of select="key('term-ids', @refid)[1]/@xreftext"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="key('term-ids', @refid)[1]"/>
        </xsl:otherwise>
      </xsl:choose>
    </a>
  </xsl:template>

  <xsl:template match="seealso">
    <b>
      <xsl:text>See also: </xsl:text>
    </b>
    <xsl:call-template name="resolveIDREFS">
      <xsl:with-param name="stringToTokenize" select="@refids"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="resolveIDREFS">
    <xsl:param name="stringToTokenize"/>
    <xsl:variable name="normalizedString">
      <xsl:value-of select="concat(normalize-space($stringToTokenize), ' ')"/>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$normalizedString!=' '">
        <xsl:variable name="firstOfString" 
          select="substring-before($normalizedString, ' ')"/>
        <xsl:variable name="restOfString" 
          select="substring-after($normalizedString, ' ')"/>
        <a href="#{$firstOfString}">
          <xsl:choose>
            <xsl:when test="key('term-ids', $firstOfString)[1]/@xreftext">
              <xsl:value-of 
                select="key('term-ids', $firstOfString)[1]/@xreftext"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="key('term-ids', $firstOfString)[1]"/>
            </xsl:otherwise>
          </xsl:choose>
        </a>
        <xsl:if test="$restOfString!=''">
          <xsl:text>, </xsl:text>
        </xsl:if>
        <xsl:call-template name="resolveIDREFS">
          <xsl:with-param name="stringToTokenize" select="$restOfString"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>.</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
